︠4cb92b21-e525-40eb-9677-8093f52a9eb7i︠
%html
<p>Montgomery curve group operations</p>

︡1a87f8b8-278b-4b0b-878c-6f7a55275942︡{"html": "<p>Montgomery curve group operations</p>"}︡
︠c8936934-7191-4366-87bb-57fe433d9629s︠
def madd(P,Q,R):
    """Montgomery addition: returns (x_{m+n},z_{m+n}) given P=(x_m,z_m), Q=(x_n,z_n), and R=(x_{m-n},z_{m-n})"""
    a = P[0]-P[1]; b=P[0]+P[1]; c=Q[0]-Q[1]; d=Q[0]+Q[1]
    e=a*d; f=b*c;
    return (R[1]*(e+f)^2,R[0]*(e-f)^2)

def mdbl(P,C):
    """Montgomery doubling: returns (x_{2n},z_{2n}) given P=(x_n,z_n) and C=(A+2)/4 where By^2=x^3+Ax^2+x is the Montgomery curve equation"""
    a=P[0]+P[1]; b=P[0]-P[1]
    c=a^2; d=b^2; e=c-d
    return (c*d,e*(d+C*e))
    
def mmul(P,n,C):
    """Montgomery multiplication: returns (x_n,z_n) given P=(x_1,z_1) and C=(A+2)/4 where By^2=x^3+Ax^2+x is the Montgomery curve equation"""
    Q = [P,mdbl(P,C)]
    b=n.digits(2)
    for i in range(len(b)-2,-1,-1):
        Q[1-b[i]] = madd(Q[1],Q[0],P)
        Q[b[i]] = mdbl(Q[b[i]],C)
    return Q[0]
︡5140c638-abcb-432b-b54d-15f39702bfd9︡{"done":true}
︠13f46029-bc37-444a-902a-61e18a4ed247s︠
F=GF(random_prime(2^256,2^255))
A=F.random_element(); B=F.random_element()
C=(A+2)/4
while true:
    x = F.random_element()
    if ((x^3+A*x^2+x)/B).is_square(): break;
P=(x,1);
t=cputime()
for p in primes(0,10000): P=mmul(P,p,C)
print "Montgomery time:", cputime()-t
a4=1/B^2-A^2/(3*B^2); a6=-A^3/(27*B^3)-a4*A/(3*B)
E=EllipticCurve([a4,a6])
P=E.random_element()
t=cputime()
for p in primes(0,10000): P=p*P
print "Weierstrass time:", cputime()-t
︡62c706e2-bdf7-4ee0-9c71-8888f32553d4︡{"stdout":"Montgomery time: 0.165339\n"}︡{"stdout":"Weierstrass time: 0.688554\n"}︡{"done":true}
︠e198e5e7-d3e2-4ca5-b189-d571ddf04c07i︠
%html
<p>Single stage ECM using Montgomery curves</p>

︡90f18c10-5d70-4d87-b49a-ac680e38053d︡{"html":"<p>Single stage ECM using Montgomery curves</p>\n\n"}︡
︠1708e0b4-9fba-4dfd-bb80-8b8a10b3d781s︠
def L(a,c,N):
    z = ceil(exp(c*log(N)^a*log(log(N))^(1-a)))
    return ceil(z)

def ECM_1curve(N,p,M,B1):
    R=Integers(N)
    # generate Montgomery curve using parameterization that guarantees a point of order 3
    c=R(randint(1,10^10))
    a=6*c/(c^2+6)
    b=R(randint(1,10^10))
    A=(-3*a^4-6*a^2+1)/(4*a^3)
    B=(a^2-1)^2/(4*a*b^2)
    C=(A+2)/4
    P=(3*a/4,1)
    a4=1/B^2-A^2/(3*B^2); a6=-A^3/(27*B^3)-a4*A/(3*B)
    F=GF(p)
    E=EllipticCurve([F(a4.lift()),F(a6.lift())])
    print factor(E.cardinality())
    m=ceil(M+2*sqrt(M)+1)
    for p in primes(B1):
        q = p; r=p*q
        while r <= m: q=r; r=p*q
        P=mmul(P,q,C)
        d=gcd(N,P[1].lift())
        if d == N: print "N fail"; return 0
        if d > 1: return d
    return 0
︡a8bf0127-9f5a-4758-9a0c-c35348eb20ca︡{"done":true}
︠bb6024da-b256-47bb-b1e4-0e4df425402ds︠
k=50; n=200
q=random_prime(2^(n-k),2^(n-k-1))
p=random_prime(2^k,2^(k-1))
B=2.5*L(1/2,1/sqrt(2),2^k) # includes an empirical fudge factor
print "B =",B
print "Expected number of iterations is about", L(1/2,1/(2*sqrt(2)),2^k)
t=cputime()
i=1
while true:
    print i,
    d = ECM_1curve(p*q,p,2^50,B)
    if d: break
    i += 1
print d
print "ECM time:", cputime()-t
print p

︡a78214ef-5252-4757-84c6-6be9e4bb40b3︡{"stdout":"B = 6340.00000000000\n"}︡{"stdout":"Expected number of iterations is about 51\n"}︡{"stdout":"1 2^3 * 3 * 5 * 7 * 1567 * 2531 * 21557\n2"}︡{"stdout":" 2^3 * 3 * 47^2 * 5479 * 247241\n3"}︡{"stdout":" 2^2 * 3^3 * 53 * 12546688591\n4"}︡{"stdout":" 2^4 * 3 * 13 * 115091709323\n5"}︡{"stdout":" 2^5 * 3^4 * 11^2 * 19 * 47 * 256423\n6"}︡{"stdout":" 2^2 * 3^3 * 7 * 94996330883\n7"}︡{"stdout":" 2^2 * 3 * 5984769840877\n8"}︡{"stdout":" 2^3 * 3^5 * 36943028737\n9"}︡{"stdout":" 2^3 * 3^2 * 59 * 16906132519\n10"}︡{"stdout":" 2^4 * 3 * 5 * 17 * 1693 * 10397087\n11"}︡{"stdout":" 2^4 * 3^2 * 13 * 19 * 2019153197\n12"}︡{"stdout":" 2^5 * 3 * 37 * 263 * 751 * 102367\n13"}︡{"stdout":" 2^2 * 3 * 211 * 28363835737\n14"}︡{"stdout":" 2^7 * 3^2 * 131 * 191 * 1259 * 1979\n"}︡{"stdout":"71817240331703\n"}︡{"stdout":"ECM time: 5.743919\n"}︡{"stdout":"71817240331703\n"}︡{"done":true}
︠9405d4d4-c3a5-4e77-8568-ed6d0d442cef︠









